@@ -1,4 +1,4 @@
-// Copyright 2020 - 2021 Martin Dosch.
+// Copyright Martin Dosch.
 // Use of this source code is governed by the BSD-2-clause
 // license that can be found in the LICENSE file.
 
@@ -7,6 +7,7 @@ package main
 import (
 	"bytes"
 	"encoding/xml"
+	"errors"
 	"fmt"
 	"log"
 	"net/http"
@@ -16,25 +17,24 @@ import (
 	"strconv"
 
 	"github.com/beevik/etree"            // BSD-2-clause
-	"github.com/gabriel-vasile/mimetype" // MIT License)
+	"github.com/gabriel-vasile/mimetype" // MIT License
 	"github.com/mattn/go-xmpp"           // BSD-3-Clause
 )
 
-func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ,
-	jserver string, filePath string) string {
+func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath string) string {
 	var uploadComponent string
 	var maxFileSize int64
 
 	// Get file size
 	fileInfo, err := os.Stat(filePath)
-	if err != nil {
+	if errors.Unwrap(err) != nil {
 		log.Fatal(err)
 	}
 	fileSize := fileInfo.Size()
 
 	// Read file
 	buffer, err := readFile(filePath)
-	if err != nil {
+	if errors.Unwrap(err) != nil {
 		log.Fatal(err)
 	}
 
@@ -45,23 +45,20 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ,
 
 	// Get file name
 	fileName := filepath.Base(filePath)
-	// Just use alphanumerical characters for now to work around
-	// https://github.com/mattn/go-xmpp/issues/132
-	reg, err := regexp.Compile(`[^a-zA-Z0-9\+\-\_\.]+`)
-	if err != nil {
-		log.Fatal(err)
-	}
+	// Just use alphanumerical and some special characters for now
+	// to work around https://github.com/mattn/go-xmpp/issues/132
+	reg := regexp.MustCompile(`[^a-zA-Z0-9\+\-\_\.]+`)
 	fileNameEscaped := reg.ReplaceAllString(fileName, "_")
 
 	// Query server for disco#items
 	iqContent, err := sendIQ(client, iqc, jserver, "get",
 		"<query xmlns='http://jabber.org/protocol/disco#items'/>")
-	if err != nil {
+	if errors.Unwrap(err) != nil {
 		log.Fatal(err)
 	}
 	iqDiscoItemsXML := etree.NewDocument()
 	err = iqDiscoItemsXML.ReadFromBytes(iqContent.Query)
-	if err != nil {
+	if errors.Unwrap(err) != nil {
 		log.Fatal(err)
 	}
 	iqDiscoItemsXMLQuery := iqDiscoItemsXML.SelectElement("query")
@@ -74,22 +71,23 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ,
 	for _, r := range iqDiscoItemsXMLItems {
 		jid := r.SelectAttr("jid")
 		iqDiscoInfoReqXML := etree.NewDocument()
+		iqDiscoInfoReqXML.WriteSettings.AttrSingleQuote = true
 		iqDiscoInfoReqXMLQuery := iqDiscoInfoReqXML.CreateElement("query")
 		iqDiscoInfoReqXMLQuery.CreateAttr("xmlns", nsDiscoInfo)
 		iqdi, err := iqDiscoInfoReqXML.WriteToString()
-		if err != nil {
+		if errors.Unwrap(err) != nil {
 			log.Fatal(err)
 		}
 		iqDiscoInfo, err := sendIQ(client, iqc, jid.Value, "get", iqdi)
-		if err != nil {
+		if errors.Unwrap(err) != nil {
 			log.Fatal(err)
 		}
-		if iqDiscoInfo.Type != "result" {
+		if iqDiscoInfo.Type != strResult {
 			continue
 		}
 		iqDiscoInfoXML := etree.NewDocument()
 		err = iqDiscoInfoXML.ReadFromBytes(iqDiscoInfo.Query)
-		if err != nil {
+		if errors.Unwrap(err) != nil {
 			log.Fatal(err)
 		}
 		iqDiscoInfoXMLQuery := iqDiscoInfoXML.SelectElement("query")
@@ -112,7 +110,6 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ,
 			iqDiscoInfoXMLCategory.Value == "store" {
 			uploadComponent = jid.Value
 		}
-
 	}
 	if uploadComponent == "" {
 		log.Fatal("No http upload component found.")
@@ -135,8 +132,8 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ,
 			}
 			if varAttr.Value == "max-file-size" && prevFieldVal.Text() == nsHTTPUpload {
 				maxFileSize, err = strconv.ParseInt(curFieldVal.Text(), 10, 64)
-				if err != nil {
-					log.Fatal("Error while checking server maximum http upload file size.")
+				if errors.Unwrap(err) != nil {
+					log.Fatal("error while checking server maximum http upload file size.")
 				}
 			}
 		}
@@ -153,27 +150,28 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ,
 	}
 
 	request := etree.NewDocument()
+	request.WriteSettings.AttrSingleQuote = true
 	requestReq := request.CreateElement("request")
 	requestReq.CreateAttr("xmlns", nsHTTPUpload)
 	requestReq.CreateAttr("filename", fileNameEscaped)
 	requestReq.CreateAttr("size", fmt.Sprint(fileSize))
 	requestReq.CreateAttr("content-type", mimeType)
 	r, err := request.WriteToString()
-	if err != nil {
+	if errors.Unwrap(err) != nil {
 		log.Fatal(err)
 	}
 
 	// Request http upload slot
 	uploadSlot, err := sendIQ(client, iqc, uploadComponent, "get", r)
-	if err != nil {
+	if errors.Unwrap(err) != nil {
 		log.Fatal(err)
 	}
-	if uploadSlot.Type != "result" {
-		log.Fatal("Error while requesting upload slot.")
+	if uploadSlot.Type != strResult {
+		log.Fatal("error while requesting upload slot.")
 	}
 	iqHTTPUploadSlotXML := etree.NewDocument()
 	err = iqHTTPUploadSlotXML.ReadFromBytes(uploadSlot.Query)
-	if err != nil {
+	if errors.Unwrap(err) != nil {
 		log.Fatal(err)
 	}
 	iqHTTPUploadSlotXMLSlot := iqHTTPUploadSlotXML.SelectElement("slot")
@@ -193,7 +191,7 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ,
 	httpClient := &http.Client{}
 	req, err := http.NewRequest(http.MethodPut, iqHTTPUploadSlotXMLPutURL.Value,
 		buffer)
-	if err != nil {
+	if errors.Unwrap(err) != nil {
 		log.Fatal(err)
 	}
 	req.Header.Set("Content-Type", mimeTypeEscaped.String())
@@ -209,7 +207,7 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ,
 		}
 	}
 	resp, err := httpClient.Do(req)
-	if err != nil {
+	if errors.Unwrap(err) != nil {
 		log.Fatal(err)
 	}
 	// Test for http status code "200 OK" or "201 Created"
@@ -226,5 +224,9 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ,
 	if iqHTTPUploadSlotXMLGetURL == nil {
 		log.Fatal("http-upload: no url attribute")
 	}
+	err = resp.Body.Close()
+	if errors.Unwrap(err) != nil {
+		fmt.Println("error while closing http request body:", err)
+	}
 	return iqHTTPUploadSlotXMLGetURL.Value
 }
