@@ -601,7 +601,8 @@ func main() {
 		}
 	}
 
-	// Wait for a short time as some messages are not delievered by the server
+	// Wait for a short time as some messages are not delivered by the server
 	// if the connection is closed immediately after sending a message.
 	time.Sleep(defaultSleepTime * time.Millisecond)
+	client.Close()
 }
