@@ -1,4 +1,4 @@
-// Copyright 2022 Martin Dosch.
+// Copyright Martin Dosch.
 // Use of this source code is governed by the BSD-2-clause
 // license that can be found in the LICENSE file.
 
@@ -7,6 +7,7 @@ package main
 import (
 	"bufio"
 	"errors"
+	"fmt"
 	"log"
 	"os"
 	"os/exec"
@@ -19,8 +20,8 @@ import (
 func findConfig() (string, error) {
 	// Get the current user.
 	curUser, err := user.Current()
-	if err != nil {
-		return "", err
+	if errors.Unwrap(err) != nil {
+		return "", fmt.Errorf("findConfig: failed to get current user: %w", err)
 	}
 	// Get home directory.
 	home := curUser.HomeDir
@@ -31,14 +32,16 @@ func findConfig() (string, error) {
 	if osConfigDir == "" {
 		osConfigDir = home + "/.config"
 	}
-	configFiles := [3]string{osConfigDir + "/go-sendxmpp/config",
+	configFiles := [3]string{
+		osConfigDir + "/go-sendxmpp/config",
 		osConfigDir + "/go-sendxmpp/sendxmpprc", home +
-			"/.sendxmpprc"}
+			"/.sendxmpprc",
+	}
 
 	for _, r := range configFiles {
 		// Check that the config file is existing.
 		_, err := os.Stat(r)
-		if err == nil {
+		if errors.Unwrap(err) == nil {
 			return r, nil
 		}
 	}
@@ -48,7 +51,6 @@ func findConfig() (string, error) {
 // Opens the config file and returns the specified values
 // for username, server and port.
 func parseConfig(configPath string) (configuration, error) {
-
 	var (
 		output configuration
 		err    error
@@ -60,7 +62,7 @@ func parseConfig(configPath string) (configuration, error) {
 	// Get systems user config path.
 	if configPath == "" {
 		configPath, err = findConfig()
-		if err != nil {
+		if errors.Unwrap(err) != nil {
 			log.Fatal(err)
 		}
 	}
@@ -68,7 +70,7 @@ func parseConfig(configPath string) (configuration, error) {
 	// Only check file permissions if we are not running on windows.
 	if runtime.GOOS != "windows" {
 		info, err := os.Stat(configPath)
-		if err != nil {
+		if errors.Unwrap(err) != nil {
 			log.Fatal(err)
 		}
 		// Check for file permissions. Must be 600, 640, 440 or 400.
@@ -82,8 +84,8 @@ func parseConfig(configPath string) (configuration, error) {
 
 	// Open config file.
 	file, err := os.Open(configPath)
-	if err != nil {
-		return output, err
+	if errors.Unwrap(err) != nil {
+		return output, fmt.Errorf("parseConfig: failed to open config file: %w", err)
 	}
 	scanner := bufio.NewScanner(file)
 	scanner.Split(bufio.ScanLines)
@@ -94,7 +96,7 @@ func parseConfig(configPath string) (configuration, error) {
 			continue
 		}
 
-		row := strings.SplitN(scanner.Text(), " ", 2)
+		row := strings.SplitN(scanner.Text(), " ", defaultConfigRowSep)
 
 		switch row[0] {
 		case "username:":
@@ -109,7 +111,7 @@ func parseConfig(configPath string) (configuration, error) {
 				shell = "/bin/sh"
 			}
 			out, err := exec.Command(shell, "-c", row[1]).Output()
-			if err != nil {
+			if errors.Unwrap(err) != nil {
 				log.Fatal(err)
 			}
 			output.password = string(out)
@@ -120,8 +122,11 @@ func parseConfig(configPath string) (configuration, error) {
 			output.port = row[1]
 		case "resource:":
 			output.resource = row[1]
+			fmt.Println(configPath+":", "Deprecated option: resource.")
+		case "alias:":
+			output.alias = row[1]
 		default:
-			if len(row) >= 2 {
+			if len(row) >= defaultConfigRowSep {
 				if strings.Contains(scanner.Text(), ";") {
 					output.username = strings.Split(row[0], ";")[0]
 					output.jserver = strings.Split(row[0], ";")[1]
@@ -129,7 +134,7 @@ func parseConfig(configPath string) (configuration, error) {
 				} else {
 					output.username = strings.Split(row[0], ":")[0]
 					jserver := strings.Split(row[0], "@")
-					if len(jserver) < 2 {
+					if len(jserver) < defaultLenServerConf {
 						log.Fatal("Couldn't parse config: ", row[0])
 					}
 					output.jserver = jserver[0]
@@ -138,17 +143,20 @@ func parseConfig(configPath string) (configuration, error) {
 			}
 		}
 	}
-	file.Close()
+	err = file.Close()
+	if errors.Unwrap(err) != nil {
+		fmt.Println("error closing file:", err)
+	}
 
 	// Check if the username is a valid JID
 	output.username, err = MarshalJID(output.username)
-	if err != nil {
+	if errors.Unwrap(err) != nil {
 		// Check whether only the local part was used by appending an @ and the
 		// server part.
 		output.username = output.username + "@" + output.jserver
 		// Check if the username is a valid JID now
 		output.username, err = MarshalJID(output.username)
-		if err != nil {
+		if errors.Unwrap(err) != nil {
 			return output, errors.New("invalid username/JID: " + output.username)
 		}
 	}
